@@ -26,11 +26,7 @@
 #include "libwbclient/wbclient.h"
 #include "lib/popt/popt.h"
 #include "../libcli/auth/libcli_auth.h"
-#if (_SAMBA_BUILD_) >= 4
 #include "lib/cmdline/popt_common.h"
-#else
-#include "popt_common.h"
-#endif
 
 #ifdef DBGC_CLASS
 #undef DBGC_CLASS
@@ -121,7 +117,8 @@ static bool parse_wbinfo_domain_user(const char *domuser, fstring domain,
 
 	if (!p) {
 		/* Maybe it was a UPN? */
-		if ((p = strchr(domuser, '@')) != NULL) {
+		p = strchr(domuser, '@');
+		if (p != NULL) {
 			fstrcpy(domain, "");
 			fstrcpy(user, domuser);
 			return true;
@@ -242,6 +239,8 @@ static bool wbinfo_get_user_sidinfo(const char *sid_str)
 		 pwd->pw_dir,
 		 pwd->pw_shell);
 
+	wbcFreeMemory(pwd);
+
 	return true;
 }
 
@@ -1226,6 +1225,9 @@ static bool wbinfo_lookupsid(const char *sid_str)
 	d_printf("%s%c%s %d\n",
 		 domain, winbind_separator(), name, type);
 
+	wbcFreeMemory(domain);
+	wbcFreeMemory(name);
+
 	return true;
 }
 
@@ -1260,6 +1262,9 @@ static bool wbinfo_lookupsid_fullname(const char *sid_str)
 	d_printf("%s%c%s %d\n",
 		 domain, winbind_separator(), name, type);
 
+	wbcFreeMemory(domain);
+	wbcFreeMemory(name);
+
 	return true;
 }
 
@@ -1452,7 +1457,8 @@ static char *wbinfo_prompt_pass(TALLOC_CTX *mem_ctx,
 				const char *username)
 {
 	char *prompt;
-	const char *ret = NULL;
+	char buf[1024] = {0};
+	int rc;
 
 	prompt = talloc_asprintf(mem_ctx, "Enter %s's ", username);
 	if (!prompt) {
@@ -1469,10 +1475,13 @@ static char *wbinfo_prompt_pass(TALLOC_CTX *mem_ctx,
 		return NULL;
 	}
 
-	ret = getpass(prompt);
+	rc = samba_getpass(prompt, buf, sizeof(buf), false, false);
 	TALLOC_FREE(prompt);
+	if (rc < 0) {
+		return NULL;
+	}
 
-	return talloc_strdup(mem_ctx, ret);
+	return talloc_strdup(mem_ctx, buf);
 }
 
 /* Authenticate a user with a plaintext password */
@@ -1876,7 +1885,10 @@ static bool wbinfo_klog(char *username)
 		*p = '%';
 	} else {
 		fstrcpy(request.data.auth.user, username);
-		fstrcpy(request.data.auth.pass, getpass("Password: "));
+		(void) samba_getpass("Password: ",
+				     request.data.auth.pass,
+				     sizeof(request.data.auth.pass),
+				     false, false);
 	}
 
 	request.flags |= WBFLAG_PAM_AFS_TOKEN;
@@ -2069,7 +2081,8 @@ enum {
 	OPT_LOGOFF,
 	OPT_LOGOFF_USER,
 	OPT_LOGOFF_UID,
-	OPT_LANMAN
+	OPT_LANMAN,
+	OPT_KRB5CCNAME
 };
 
 int main(int argc, char **argv, char **envp)
@@ -2170,7 +2183,7 @@ int main(int argc, char **argv, char **envp)
 		{ "krb5auth", 'K', POPT_ARG_STRING, &string_arg, 'K', "authenticate user using Kerberos", "user%password" },
 			/* destroys wbinfo --help output */
 			/* "user%password,DOM\\user%password,user@EXAMPLE.COM,EXAMPLE.COM\\user%password" }, */
-		{ "krb5ccname", 0, POPT_ARG_STRING, &opt_krb5ccname, '0', "authenticate user using Kerberos and specific credential cache type", "krb5ccname" },
+		{ "krb5ccname", 0, POPT_ARG_STRING, &opt_krb5ccname, OPT_KRB5CCNAME, "authenticate user using Kerberos and specific credential cache type", "krb5ccname" },
 #endif
 		{ "separator", 0, POPT_ARG_NONE, 0, OPT_SEPARATOR, "Get the active winbind separator", NULL },
 		{ "verbose", 0, POPT_ARG_NONE, 0, OPT_VERBOSE, "Print additional information per command", NULL },
@@ -2626,6 +2639,7 @@ int main(int argc, char **argv, char **envp)
 		case OPT_LANMAN:
 		case OPT_LOGOFF_USER:
 		case OPT_LOGOFF_UID:
+		case OPT_KRB5CCNAME:
 			break;
 		default:
 			d_fprintf(stderr, "Invalid option\n");
