@@ -358,26 +358,6 @@ static ADS_STATUS libnet_join_find_machine_acct(TALLOC_CTX *mem_ctx,
 	return status;
 }
 
-static ADS_STATUS libnet_join_get_machine_spns(TALLOC_CTX *mem_ctx,
-					       struct libnet_JoinCtx *r,
-					       char ***spn_array,
-					       size_t *num_spns)
-{
-	ADS_STATUS status;
-
-	if (r->in.machine_name == NULL) {
-		return ADS_ERROR_SYSTEM(EINVAL);
-	}
-
-	status = ads_get_service_principal_names(mem_ctx,
-						 r->in.ads,
-						 r->in.machine_name,
-						 spn_array,
-						 num_spns);
-
-	return status;
-}
-
 /****************************************************************
  Set a machines dNSHostName and servicePrincipalName attributes
 ****************************************************************/
@@ -388,10 +368,8 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 	ADS_STATUS status;
 	ADS_MODLIST mods;
 	fstring my_fqdn;
-	const char **spn_array = NULL;
-	size_t num_spns = 0;
+	const char *spn_array[3] = {NULL, NULL, NULL};
 	char *spn = NULL;
-	bool ok;
 
 	/* Find our DN */
 
@@ -400,14 +378,6 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		return status;
 	}
 
-	status = libnet_join_get_machine_spns(mem_ctx,
-					      r,
-					      discard_const_p(char **, &spn_array),
-					      &num_spns);
-	if (!ADS_ERR_OK(status)) {
-		DEBUG(5, ("Retrieving the servicePrincipalNames failed.\n"));
-	}
-
 	/* Windows only creates HOST/shortname & HOST/fqdn. */
 
 	spn = talloc_asprintf(mem_ctx, "HOST/%s", r->in.machine_name);
@@ -417,15 +387,7 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 	if (!strupper_m(spn)) {
 		return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
 	}
-
-	ok = ads_element_in_array(spn_array, num_spns, spn);
-	if (!ok) {
-		ok = add_string_to_array(spn_array, spn,
-					 &spn_array, (int *)&num_spns);
-		if (!ok) {
-			return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
-		}
-	}
+	spn_array[0] = spn;
 
 	if (!name_to_fqdn(my_fqdn, r->in.machine_name)
 	    || (strchr(my_fqdn, '.') == NULL)) {
@@ -442,24 +404,9 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		if (!spn) {
 			return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
 		}
-
-		ok = ads_element_in_array(spn_array, num_spns, spn);
-		if (!ok) {
-			ok = add_string_to_array(spn_array, spn,
-						 &spn_array, (int *)&num_spns);
-			if (!ok) {
-				return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
-			}
-		}
+		spn_array[1] = spn;
 	}
 
-	/* make sure to NULL terminate the array */
-	spn_array = talloc_realloc(mem_ctx, spn_array, const char *, num_spns + 1);
-	if (spn_array == NULL) {
-		return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
-	}
-	spn_array[num_spns] = NULL;
-
 	mods = ads_init_mods(mem_ctx);
 	if (!mods) {
 		return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
@@ -916,6 +863,7 @@ static NTSTATUS libnet_join_joindomain_rpc(TALLOC_CTX *mem_ctx,
 	struct samr_Ids name_types;
 	union samr_UserInfo user_info;
 	struct dcerpc_binding_handle *b = NULL;
+	unsigned int old_timeout = 0;
 
 	DATA_BLOB session_key = data_blob_null;
 	struct samr_CryptPassword crypt_pwd;
@@ -1148,6 +1096,12 @@ static NTSTATUS libnet_join_joindomain_rpc(TALLOC_CTX *mem_ctx,
 
 	/* Set password on machine account - first try level 26 */
 
+	/*
+	 * increase the timeout as password filter modules on the DC
+	 * might delay the operation for a significant amount of time
+	 */
+	old_timeout = rpccli_set_timeout(pipe_hnd, 600000);
+
 	init_samr_CryptPasswordEx(r->in.machine_password,
 				  &session_key,
 				  &crypt_pwd_ex);
@@ -1179,6 +1133,8 @@ static NTSTATUS libnet_join_joindomain_rpc(TALLOC_CTX *mem_ctx,
 						  &result);
 	}
 
+	old_timeout = rpccli_set_timeout(pipe_hnd, old_timeout);
+
 	if (!NT_STATUS_IS_OK(status)) {
 
 		dcerpc_samr_DeleteUser(b, mem_ctx,
