@@ -19,7 +19,7 @@
 
 #include "includes.h"
 #include "winbindd.h"
-#include "librpc/gen_ndr/cli_wbint.h"
+#include "librpc/gen_ndr/ndr_wbint_c.h"
 
 struct wb_fill_pwent_state {
 	struct tevent_context *ev;
@@ -54,7 +54,7 @@ struct tevent_req *wb_fill_pwent_send(TALLOC_CTX *mem_ctx,
 	state->info = info;
 	state->pw = pw;
 
-	subreq = wb_sid2uid_send(state, state->ev, &state->info->user_sid);
+	subreq = wb_sids2xids_send(state, state->ev, &state->info->user_sid, 1);
 	if (tevent_req_nomem(subreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -69,15 +69,28 @@ static void wb_fill_pwent_sid2uid_done(struct tevent_req *subreq)
 	struct wb_fill_pwent_state *state = tevent_req_data(
 		req, struct wb_fill_pwent_state);
 	NTSTATUS status;
+	struct unixid xid;
 
-	status = wb_sid2uid_recv(subreq, &state->pw->pw_uid);
+	status = wb_sids2xids_recv(subreq, &xid);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 
-	subreq = wb_sid2gid_send(state, state->ev, &state->info->group_sid);
+	/*
+	 * We are filtering further down in sids2xids, but that filtering
+	 * depends on the actual type of the sid handed in (as determined
+	 * by lookupsids). Here we need to filter for the type of object
+	 * actually requested, in this case uid.
+	 */
+	if (!(xid.type == ID_TYPE_UID || xid.type == ID_TYPE_BOTH)) {
+		tevent_req_nterror(req, NT_STATUS_NONE_MAPPED);
+		return;
+	}
+
+	state->pw->pw_uid = (uid_t)xid.id;
+
+	subreq = wb_sids2xids_send(state, state->ev, &state->info->group_sid, 1);
 	if (tevent_req_nomem(subreq, req)) {
 		return;
 	}
@@ -95,14 +108,27 @@ static void wb_fill_pwent_sid2gid_done(struct tevent_req *subreq)
 	fstring user_name, output_username;
 	char *mapped_name = NULL;
 	NTSTATUS status;
+	struct unixid xid;
 
-	status = wb_sid2gid_recv(subreq, &state->pw->pw_gid);
+	status = wb_sids2xids_recv(subreq, &xid);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
-		tevent_req_nterror(req, status);
+	if (tevent_req_nterror(req, status)) {
 		return;
 	}
 
+	/*
+	 * We are filtering further down in sids2xids, but that filtering
+	 * depends on the actual type of the sid handed in (as determined
+	 * by lookupsids). Here we need to filter for the type of object
+	 * actually requested, in this case gid.
+	 */
+	if (!(xid.type == ID_TYPE_GID || xid.type == ID_TYPE_BOTH)) {
+		tevent_req_nterror(req, NT_STATUS_NONE_MAPPED);
+		return;
+	}
+
+	state->pw->pw_gid = (gid_t)xid.id;
+
 	domain = find_domain_from_sid_noinit(&state->info->user_sid);
 	if (domain == NULL) {
 		tevent_req_nterror(req, NT_STATUS_NO_SUCH_USER);
@@ -113,7 +139,10 @@ static void wb_fill_pwent_sid2gid_done(struct tevent_req *subreq)
 	/* Username */
 
 	fstrcpy(user_name, state->info->acct_name);
-	strlower_m(user_name);
+	if (!strlower_m(user_name)) {
+		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
+		return;
+	}
 	status = normalize_name_map(state, domain, user_name, &mapped_name);
 
 	/* Basic removal of whitespace */
@@ -131,7 +160,9 @@ static void wb_fill_pwent_sid2gid_done(struct tevent_req *subreq)
 				     true);
 	}
 
-	fstrcpy(state->pw->pw_name, output_username);
+	strlcpy(state->pw->pw_name,
+		output_username,
+		sizeof(state->pw->pw_name));
 	fstrcpy(state->pw->pw_gecos, state->info->full_name);
 
 	/* Home directory and shell */
@@ -194,7 +225,7 @@ static bool fillup_pw_field(const char *lp_template,
 	if (!templ)
 		return False;
 
-	safe_strcpy(out, templ, sizeof(fstring) - 1);
+	strlcpy(out, templ, sizeof(fstring));
 	TALLOC_FREE(templ);
 
 	return True;
