@@ -369,8 +369,26 @@ static void wb_domain_request_initialized(struct tevent_req *subreq)
 		tevent_req_error(req, EINVAL);
 		return;
 	}
-	fstrcpy(state->domain->name, response->data.domain_info.name);
-	fstrcpy(state->domain->alt_name, response->data.domain_info.alt_name);
+
+	talloc_free(state->domain->name);
+	state->domain->name = talloc_strdup(state->domain,
+					    response->data.domain_info.name);
+	if (state->domain->name == NULL) {
+		tevent_req_error(req, ENOMEM);
+		return;
+	}
+
+	if (response->data.domain_info.alt_name[0] != '\0') {
+		talloc_free(state->domain->alt_name);
+
+		state->domain->alt_name = talloc_strdup(state->domain,
+				response->data.domain_info.alt_name);
+		if (state->domain->alt_name == NULL) {
+			tevent_req_error(req, ENOMEM);
+			return;
+		}
+	}
+
 	state->domain->native_mode = response->data.domain_info.native_mode;
 	state->domain->active_directory =
 		response->data.domain_info.active_directory;
@@ -524,7 +542,7 @@ void winbind_child_died(pid_t pid)
 void winbindd_flush_negative_conn_cache(struct winbindd_domain *domain)
 {
 	flush_negative_conn_cache_for_domain(domain->name);
-	if (*domain->alt_name) {
+	if (domain->alt_name != NULL) {
 		flush_negative_conn_cache_for_domain(domain->alt_name);
 	}
 }
@@ -612,7 +630,7 @@ void winbind_msg_offline(struct messaging_context *msg_ctx,
 
 		messaging_send_buf(msg_ctx, pid_to_procid(child->pid),
 				   MSG_WINBIND_OFFLINE,
-				   (uint8 *)child->domain->name,
+				   (const uint8_t *)child->domain->name,
 				   strlen(child->domain->name)+1);
 	}
 }
@@ -661,7 +679,7 @@ void winbind_msg_online(struct messaging_context *msg_ctx,
 				messaging_send_buf(msg_ctx,
 						   pid_to_procid(idmap->pid), 
 						   MSG_WINBIND_ONLINE,
-						   (uint8 *)domain->name,
+						   (const uint8_t *)domain->name,
 						   strlen(domain->name)+1);
 			}
 		}
@@ -686,7 +704,7 @@ void winbind_msg_online(struct messaging_context *msg_ctx,
 
 		messaging_send_buf(msg_ctx, pid_to_procid(child->pid),
 				   MSG_WINBIND_ONLINE,
-				   (uint8 *)child->domain->name,
+				   (const uint8_t *)child->domain->name,
 				   strlen(child->domain->name)+1);
 	}
 }
@@ -859,8 +877,8 @@ void winbind_msg_dump_domain_list(struct messaging_context *msg_ctx,
 	talloc_destroy(mem_ctx);
 }
 
-static void account_lockout_policy_handler(struct event_context *ctx,
-					   struct timed_event *te,
+static void account_lockout_policy_handler(struct tevent_context *ctx,
+					   struct tevent_timer *te,
 					   struct timeval now,
 					   void *private_data)
 {
@@ -898,7 +916,7 @@ static void account_lockout_policy_handler(struct event_context *ctx,
 			 nt_errstr(result)));
 	}
 
-	child->lockout_policy_event = event_add_timed(winbind_event_context(), NULL,
+	child->lockout_policy_event = tevent_add_timer(winbind_event_context(), NULL,
 						      timeval_current_ofs(3600, 0),
 						      account_lockout_policy_handler,
 						      child);
@@ -976,8 +994,8 @@ static bool calculate_next_machine_pwd_change(const char *domain,
 	return true;
 }
 
-static void machine_password_change_handler(struct event_context *ctx,
-					    struct timed_event *te,
+static void machine_password_change_handler(struct tevent_context *ctx,
+					    struct tevent_timer *te,
 					    struct timeval now,
 					    void *private_data)
 {
@@ -1060,7 +1078,7 @@ static void machine_password_change_handler(struct event_context *ctx,
 	}
 
 done:
-	child->machine_password_change_event = event_add_timed(winbind_event_context(), NULL,
+	child->machine_password_change_event = tevent_add_timer(winbind_event_context(), NULL,
 							      next_change,
 							      machine_password_change_handler,
 							      child);
@@ -1294,15 +1312,73 @@ struct winbindd_domain *wb_child_domain(void)
 	return child_domain;
 }
 
+struct child_handler_state {
+	struct winbindd_child *child;
+	struct winbindd_cli_state cli;
+};
+
+static void child_handler(struct tevent_context *ev, struct tevent_fd *fde,
+			  uint16_t flags, void *private_data)
+{
+	struct child_handler_state *state =
+		(struct child_handler_state *)private_data;
+	NTSTATUS status;
+	struct iovec iov[2];
+	int iov_count;
+
+	/* fetch a request from the main daemon */
+	status = child_read_request(&state->cli);
+
+	if (!NT_STATUS_IS_OK(status)) {
+		/* we lost contact with our parent */
+		_exit(0);
+	}
+
+	DEBUG(4,("child daemon request %d\n",
+		 (int)state->cli.request->cmd));
+
+	ZERO_STRUCTP(state->cli.response);
+	state->cli.request->null_term = '\0';
+	state->cli.mem_ctx = talloc_tos();
+	child_process_request(state->child, &state->cli);
+
+	DEBUG(4, ("Finished processing child request %d\n",
+		  (int)state->cli.request->cmd));
+
+	SAFE_FREE(state->cli.request->extra_data.data);
+
+	iov[0].iov_base = (void *)state->cli.response;
+	iov[0].iov_len = sizeof(struct winbindd_response);
+	iov_count = 1;
+
+	if (state->cli.response->length >
+	    sizeof(struct winbindd_response)) {
+		iov[1].iov_base =
+			(void *)state->cli.response->extra_data.data;
+		iov[1].iov_len = state->cli.response->length-iov[0].iov_len;
+		iov_count = 2;
+	}
+
+	DEBUG(10, ("Writing %d bytes to parent\n",
+		   (int)state->cli.response->length));
+
+	if (write_data_iov(state->cli.sock, iov, iov_count) !=
+	    state->cli.response->length) {
+		DEBUG(0, ("Could not write result\n"));
+		exit(1);
+	}
+}
+
 static bool fork_domain_child(struct winbindd_child *child)
 {
 	int fdpair[2];
-	struct winbindd_cli_state state;
+	struct child_handler_state state;
 	struct winbindd_request request;
 	struct winbindd_response response;
 	struct winbindd_domain *primary_domain = NULL;
 	NTSTATUS status;
 	ssize_t nwritten;
+	struct tevent_fd *fde;
 
 	if (child->domain) {
 		DEBUG(10, ("fork_domain_child called for domain '%s'\n",
@@ -1318,14 +1394,17 @@ static bool fork_domain_child(struct winbindd_child *child)
 	}
 
 	ZERO_STRUCT(state);
-	state.pid = getpid();
-	state.request = &request;
-	state.response = &response;
+	state.child = child;
+	state.cli.pid = getpid();
+	state.cli.request = &request;
+	state.cli.response = &response;
 
 	child->pid = fork();
 
 	if (child->pid == -1) {
 		DEBUG(0, ("Could not fork: %s\n", strerror(errno)));
+		close(fdpair[0]);
+		close(fdpair[1]);
 		return False;
 	}
 
@@ -1361,12 +1440,12 @@ static bool fork_domain_child(struct winbindd_child *child)
 
 	DEBUG(10, ("Child process %d\n", (int)getpid()));
 
-	state.sock = fdpair[0];
+	state.cli.sock = fdpair[0];
 	close(fdpair[1]);
 
 	status = winbindd_reinit_after_fork(child, child->logfilename);
 
-	nwritten = sys_write(state.sock, &status, sizeof(status));
+	nwritten = sys_write(state.cli.sock, &status, sizeof(status));
 	if (nwritten != sizeof(status)) {
 		DEBUG(1, ("fork_domain_child: Could not write status: "
 			  "nwritten=%d, error=%s\n", (int)nwritten,
@@ -1443,7 +1522,7 @@ static bool fork_domain_child(struct winbindd_child *child)
 			set_domain_online_request(primary_domain);
 		}
 
-		child->lockout_policy_event = event_add_timed(
+		child->lockout_policy_event = tevent_add_timer(
 			winbind_event_context(), NULL, timeval_zero(),
 			account_lockout_policy_handler,
 			child);
@@ -1457,28 +1536,30 @@ static bool fork_domain_child(struct winbindd_child *child)
 
 		if (calculate_next_machine_pwd_change(child->domain->name,
 						       &next_change)) {
-			child->machine_password_change_event = event_add_timed(
+			child->machine_password_change_event = tevent_add_timer(
 				winbind_event_context(), NULL, next_change,
 				machine_password_change_handler,
 				child);
 		}
 	}
 
+	fde = tevent_add_fd(winbind_event_context(), NULL, state.cli.sock,
+			    TEVENT_FD_READ, child_handler, &state);
+	if (fde == NULL) {
+		DEBUG(1, ("tevent_add_fd failed\n"));
+		_exit(1);
+	}
+
 	while (1) {
 
 		int ret;
-		struct pollfd *pfds;
-		int num_pfds;
-		int timeout;
-		struct timeval t;
-		struct timeval *tp;
 		TALLOC_CTX *frame = talloc_stackframe();
-		struct iovec iov[2];
-		int iov_count;
 
-		if (run_events_poll(winbind_event_context(), 0, NULL, 0)) {
-			TALLOC_FREE(frame);
-			continue;
+		ret = tevent_loop_once(winbind_event_context());
+		if (ret != 0) {
+			DEBUG(1, ("tevent_loop_once failed: %s\n",
+				  strerror(errno)));
+			_exit(1);
 		}
 
 		if (child->domain && child->domain->startup &&
@@ -1489,99 +1570,6 @@ static bool fork_domain_child(struct winbindd_child *child)
 			child->domain->startup = False;
 		}
 
-		pfds = talloc_zero(talloc_tos(), struct pollfd);
-		if (pfds == NULL) {
-			DEBUG(1, ("talloc failed\n"));
-			_exit(1);
-		}
-
-		pfds->fd = state.sock;
-		pfds->events = POLLIN|POLLHUP;
-		num_pfds = 1;
-
-		timeout = INT_MAX;
-
-		if (!event_add_to_poll_args(
-			    winbind_event_context(), talloc_tos(),
-			    &pfds, &num_pfds, &timeout)) {
-			DEBUG(1, ("event_add_to_poll_args failed\n"));
-			_exit(1);
-		}
-		tp = get_timed_events_timeout(winbind_event_context(), &t);
-		if (tp) {
-			DEBUG(11,("select will use timeout of %u.%u seconds\n",
-				(unsigned int)tp->tv_sec, (unsigned int)tp->tv_usec ));
-		}
-
-		ret = poll(pfds, num_pfds, timeout);
-
-		if (run_events_poll(winbind_event_context(), ret,
-				    pfds, num_pfds)) {
-			/* We got a signal - continue. */
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		TALLOC_FREE(pfds);
-
-		if (ret == 0) {
-			DEBUG(11,("nothing is ready yet, continue\n"));
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		if (ret == -1 && errno == EINTR) {
-			/* We got a signal - continue. */
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		if (ret == -1 && errno != EINTR) {
-			DEBUG(0,("poll error occured\n"));
-			TALLOC_FREE(frame);
-			perror("poll");
-			_exit(1);
-		}
-
-		/* fetch a request from the main daemon */
-		status = child_read_request(&state);
-
-		if (!NT_STATUS_IS_OK(status)) {
-			/* we lost contact with our parent */
-			_exit(0);
-		}
-
-		DEBUG(4,("child daemon request %d\n", (int)state.request->cmd));
-
-		ZERO_STRUCTP(state.response);
-		state.request->null_term = '\0';
-		state.mem_ctx = frame;
-		child_process_request(child, &state);
-
-		DEBUG(4, ("Finished processing child request %d\n",
-			  (int)state.request->cmd));
-
-		SAFE_FREE(state.request->extra_data.data);
-
-		iov[0].iov_base = (void *)state.response;
-		iov[0].iov_len = sizeof(struct winbindd_response);
-		iov_count = 1;
-
-		if (state.response->length > sizeof(struct winbindd_response)) {
-			iov[1].iov_base =
-				(void *)state.response->extra_data.data;
-			iov[1].iov_len = state.response->length-iov[0].iov_len;
-			iov_count = 2;
-		}
-
-		DEBUG(10, ("Writing %d bytes to parent\n",
-			   (int)state.response->length));
-
-		if (write_data_iov(state.sock, iov, iov_count) !=
-		    state.response->length) {
-			DEBUG(0, ("Could not write result\n"));
-			exit(1);
-		}
 		TALLOC_FREE(frame);
 	}
 }
