@@ -65,9 +65,9 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	struct interface *ifaces;
 	bool low_port = try_low_port;
 
-	load_interfaces(tctx, lpcfg_interfaces(tctx->lp_ctx), &ifaces);
+	load_interface_list(tctx, tctx->lp_ctx, &ifaces);
 
-	myaddress = talloc_strdup(tctx, iface_best_ip(ifaces, address));
+	myaddress = talloc_strdup(tctx, iface_list_best_ip(ifaces, address));
 
 	socket_address = socket_address_from_strings(tctx, 
 						     nbtsock->sock->backend_name,
@@ -156,7 +156,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 		 */
 		req = nbt_name_register_send(nbtsock, &name_register);
 		while (true) {
-			event_loop_once(nbtsock->event_ctx);
+			tevent_loop_once(nbtsock->event_ctx);
 			if (req->state != NBT_REQUEST_WAIT) {
 				break;
 			}
@@ -170,7 +170,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 				req->state = NBT_REQUEST_SEND;
 				DLIST_ADD_END(nbtsock->send_queue, req,
 					      struct nbt_name_request *);
-				EVENT_FD_WRITEABLE(nbtsock->fde);
+				TEVENT_FD_WRITEABLE(nbtsock->fde);
 				break;
 			}
 		}
